#!/usr/bin/env vpython3

# Copyright 2017 The LibYuv Project Authors. All rights reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS. All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

import glob
import os
import shutil
import sys
import tempfile
import unittest

import roll_deps
from roll_deps import CalculateChangedDeps, GetMatchingDepsEntries, \
  ParseDepsDict, ParseLocalDepsFile, UpdateDepsFile

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
PARENT_DIR = os.path.join(SCRIPT_DIR, os.pardir)
sys.path.append(PARENT_DIR)

TEST_DATA_VARS = {
  'chromium_git': 'https://chromium.googlesource.com',
  'chromium_revision': '1b9c098a08e40114e44b6c1ec33ddf95c40b901d',
}

DEPS_ENTRIES = {
  'src/build': 'https://build.com',
  'src/buildtools': 'https://buildtools.com',
  'src/testing/gtest': 'https://gtest.com',
  'src/testing/gmock': 'https://gmock.com',
}

BUILD_OLD_REV = '52f7afeca991d96d68cf0507e20dbdd5b845691f'
BUILD_NEW_REV = 'HEAD'
BUILDTOOLS_OLD_REV = '64e38f0cebdde27aa0cfb405f330063582f9ac76'
BUILDTOOLS_NEW_REV = '55ad626b08ef971fd82a62b7abb325359542952b'


class TestError(Exception):
  pass


class FakeCmd():
  def __init__(self):
    self.expectations = []

  def add_expectation(self, *args, **kwargs):
    returns = kwargs.pop('_returns', None)
    self.expectations.append((args, kwargs, returns))

  def __call__(self, *args, **kwargs):
    if not self.expectations:
      raise TestError('Got unexpected\n%s\n%s' % (args, kwargs))
    exp_args, exp_kwargs, exp_returns = self.expectations.pop(0)
    if args != exp_args or kwargs != exp_kwargs:
      message = 'Expected:\n  args: %s\n  kwargs: %s\n' % (exp_args, exp_kwargs)
      message += 'Got:\n  args: %s\n  kwargs: %s\n' % (args, kwargs)
      raise TestError(message)
    return exp_returns


class TestRollChromiumRevision(unittest.TestCase):
  def setUp(self):
    self._output_dir = tempfile.mkdtemp()
    for test_file in glob.glob(os.path.join(SCRIPT_DIR, 'testdata', '*')):
      shutil.copy(test_file, self._output_dir)
    self._libyuv_depsfile = os.path.join(self._output_dir, 'DEPS')
    self._old_cr_depsfile = os.path.join(self._output_dir, 'DEPS.chromium.old')
    self._new_cr_depsfile = os.path.join(self._output_dir, 'DEPS.chromium.new')

    self.fake = FakeCmd()
    self.old_RunCommand = getattr(roll_deps, '_RunCommand')
    setattr(roll_deps, '_RunCommand', self.fake)

  def tearDown(self):
    shutil.rmtree(self._output_dir, ignore_errors=True)
    self.assertEqual(self.fake.expectations, [])
    setattr(roll_deps, '_RunCommand', self.old_RunCommand)

  def testVarLookup(self):
    local_scope = {'foo': 'wrong', 'vars': {'foo': 'bar'}}
    lookup = roll_deps.VarLookup(local_scope)
    self.assertEqual(lookup('foo'), 'bar')

  def testUpdateDepsFile(self):
    new_rev = 'aaaaabbbbbcccccdddddeeeeefffff0000011111'

    current_rev = TEST_DATA_VARS['chromium_revision']
    UpdateDepsFile(self._libyuv_depsfile, current_rev, new_rev, [])
    with open(self._libyuv_depsfile, 'r') as deps_file:
      deps_contents = deps_file.read()
      self.assertTrue(new_rev in deps_contents,
                      'Failed to find %s in\n%s' % (new_rev, deps_contents))

  def testParseDepsDict(self):
    with open(self._libyuv_depsfile, 'r') as deps_file:
      deps_contents = deps_file.read()
    local_scope = ParseDepsDict(deps_contents)
    vars_dict = local_scope['vars']

    def assertVar(variable_name):
      self.assertEqual(vars_dict[variable_name], TEST_DATA_VARS[variable_name])
    assertVar('chromium_git')
    assertVar('chromium_revision')
    self.assertEqual(len(local_scope['deps']), 3)

  def testGetMatchingDepsEntriesReturnsPathInSimpleCase(self):
    entries = GetMatchingDepsEntries(DEPS_ENTRIES, 'src/testing/gtest')
    self.assertEqual(len(entries), 1)
    self.assertEqual(entries[0], DEPS_ENTRIES['src/testing/gtest'])

  def testGetMatchingDepsEntriesHandlesSimilarStartingPaths(self):
    entries = GetMatchingDepsEntries(DEPS_ENTRIES, 'src/testing')
    self.assertEqual(len(entries), 2)

  def testGetMatchingDepsEntriesHandlesTwoPathsWithIdenticalFirstParts(self):
    entries = GetMatchingDepsEntries(DEPS_ENTRIES, 'src/build')
    self.assertEqual(len(entries), 1)
    self.assertEqual(entries[0], DEPS_ENTRIES['src/build'])

  def testCalculateChangedDeps(self):
    _SetupGitLsRemoteCall(self.fake,
        'https://chromium.googlesource.com/chromium/src/build', BUILD_NEW_REV)
    libyuv_deps = ParseLocalDepsFile(self._libyuv_depsfile)
    new_cr_deps = ParseLocalDepsFile(self._new_cr_depsfile)
    changed_deps = CalculateChangedDeps(libyuv_deps, new_cr_deps)
    self.assertEqual(len(changed_deps), 2)
    self.assertEqual(changed_deps[0].path, 'src/build')
    self.assertEqual(changed_deps[0].current_rev, BUILD_OLD_REV)
    self.assertEqual(changed_deps[0].new_rev, BUILD_NEW_REV)

    self.assertEqual(changed_deps[1].path, 'src/buildtools')
    self.assertEqual(changed_deps[1].current_rev, BUILDTOOLS_OLD_REV)
    self.assertEqual(changed_deps[1].new_rev, BUILDTOOLS_NEW_REV)


def _SetupGitLsRemoteCall(cmd_fake, url, revision):
  cmd = ['git', 'ls-remote', url, revision]
  cmd_fake.add_expectation(cmd, _returns=(revision, None))


if __name__ == '__main__':
  unittest.main()



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Input mixer module.
"""

import logging
import os

from . import exceptions
from . import signal_processing


class ApmInputMixer(object):
    """Class to mix a set of audio segments down to the APM input."""

    _HARD_CLIPPING_LOG_MSG = 'hard clipping detected in the  mixed signal'

    def __init__(self):
        pass

    @classmethod
    def HardClippingLogMessage(cls):
        """Returns the log message used when hard clipping is detected in the mix.

    This method is mainly intended to be used by the unit tests.
    """
        return cls._HARD_CLIPPING_LOG_MSG

    @classmethod
    def Mix(cls, output_path, capture_input_filepath, echo_filepath):
        """Mixes capture and echo.

    Creates the overall capture input for APM by mixing the "echo-free" capture
    signal with the echo signal (e.g., echo simulated via the
    echo_path_simulation module).

    The echo signal cannot be shorter than the capture signal and the generated
    mix will have the same duration of the capture signal. The latter property
    is enforced in order to let the input of APM and the reference signal
    created by TestDataGenerator have the same length (required for the
    evaluation step).

    Hard-clipping may occur in the mix; a warning is raised when this happens.

    If `echo_filepath` is None, nothing is done and `capture_input_filepath` is
    returned.

    Args:
      speech: AudioSegment instance.
      echo_path: AudioSegment instance or None.

    Returns:
      Path to the mix audio track file.
    """
        if echo_filepath is None:
            return capture_input_filepath

        # Build the mix output file name as a function of the echo file name.
        # This ensures that if the internal parameters of the echo path simulator
        # change, no erroneous cache hit occurs.
        echo_file_name, _ = os.path.splitext(os.path.split(echo_filepath)[1])
        capture_input_file_name, _ = os.path.splitext(
            os.path.split(capture_input_filepath)[1])
        mix_filepath = os.path.join(
            output_path,
            'mix_capture_{}_{}.wav'.format(capture_input_file_name,
                                           echo_file_name))

        # Create the mix if not done yet.
        mix = None
        if not os.path.exists(mix_filepath):
            echo_free_capture = signal_processing.SignalProcessingUtils.LoadWav(
                capture_input_filepath)
            echo = signal_processing.SignalProcessingUtils.LoadWav(
                echo_filepath)

            if signal_processing.SignalProcessingUtils.CountSamples(echo) < (
                    signal_processing.SignalProcessingUtils.CountSamples(
                        echo_free_capture)):
                raise exceptions.InputMixerException(
                    'echo cannot be shorter than capture')

            mix = echo_free_capture.overlay(echo)
            signal_processing.SignalProcessingUtils.SaveWav(mix_filepath, mix)

        # Check if hard clipping occurs.
        if mix is None:
            mix = signal_processing.SignalProcessingUtils.LoadWav(mix_filepath)
        if signal_processing.SignalProcessingUtils.DetectHardClipping(mix):
            logging.warning(cls._HARD_CLIPPING_LOG_MSG)

        return mix_filepath



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Unit tests for the echo path simulation module.
"""

import shutil
import os
import tempfile
import unittest

import pydub

from . import echo_path_simulation
from . import echo_path_simulation_factory
from . import signal_processing


class TestEchoPathSimulators(unittest.TestCase):
    """Unit tests for the eval_scores module.
  """

    def setUp(self):
        """Creates temporary data."""
        self._tmp_path = tempfile.mkdtemp()

        # Create and save white noise.
        silence = pydub.AudioSegment.silent(duration=1000, frame_rate=48000)
        white_noise = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            silence)
        self._audio_track_num_samples = (
            signal_processing.SignalProcessingUtils.CountSamples(white_noise))
        self._audio_track_filepath = os.path.join(self._tmp_path,
                                                  'white_noise.wav')
        signal_processing.SignalProcessingUtils.SaveWav(
            self._audio_track_filepath, white_noise)

        # Make a copy the white noise audio track file; it will be used by
        # echo_path_simulation.RecordedEchoPathSimulator.
        shutil.copy(self._audio_track_filepath,
                    os.path.join(self._tmp_path, 'white_noise_echo.wav'))

    def tearDown(self):
        """Recursively deletes temporary folders."""
        shutil.rmtree(self._tmp_path)

    def testRegisteredClasses(self):
        # Check that there is at least one registered echo path simulator.
        registered_classes = (
            echo_path_simulation.EchoPathSimulator.REGISTERED_CLASSES)
        self.assertIsInstance(registered_classes, dict)
        self.assertGreater(len(registered_classes), 0)

        # Instance factory.
        factory = echo_path_simulation_factory.EchoPathSimulatorFactory()

        # Try each registered echo path simulator.
        for echo_path_simulator_name in registered_classes:
            simulator = factory.GetInstance(
                echo_path_simulator_class=registered_classes[
                    echo_path_simulator_name],
                render_input_filepath=self._audio_track_filepath)

            echo_filepath = simulator.Simulate(self._tmp_path)
            if echo_filepath is None:
                self.assertEqual(echo_path_simulation.NoEchoPathSimulator.NAME,
                                 echo_path_simulator_name)
                # No other tests in this case.
                continue

            # Check that the echo audio track file exists and its length is greater or
            # equal to that of the render audio track.
            self.assertTrue(os.path.exists(echo_filepath))
            echo = signal_processing.SignalProcessingUtils.LoadWav(
                echo_filepath)
            self.assertGreaterEqual(
                signal_processing.SignalProcessingUtils.CountSamples(echo),
                self._audio_track_num_samples)
